package org.servlet;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import javax.servlet.ServletConfig;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @项目: SSOAuth
 * @实现: SSOAuth作为身份验证系统单独部署在了一个tomcat上，SSOWebDemo1和SSOWebDemo2是部署在同一个tomcat上。
 * @创建人: ZS
 * @创建时间: 2020/12/24
 * @描述: AuthServlet:统一认证中心
 */
public class AuthServlet extends HttpServlet {

    /**
    *调用父类的构造方法，这里调用了无参的构造方法
    */
    public AuthServlet() {
        super();
    }

    /**
    *调用父类的销毁方法,关闭tomcat时销毁servlet
    */
    public void destroy() {
        super.destroy();
        System.out.println("销毁Authservlet");
    }

    /**
     * Servlet的doGet方法。
     * 当表单的标记值方法等于get时，将调用此方法。
     * @param request 请求客户端向服务器发送请求
     * @param response 响应服务器向客户端发送的响应
     * @throws IOException 如果发生错误,引发IOException，则抛出ServletException。
    */
    public void doGet(HttpServletRequest request, HttpServletResponse response)
            throws IOException {
        doPost(request,response);
    }

    /**
     * Servlet的doPost方法。
     * 当表单的标记值方法等于post时，将调用此方法。
     * @param request 请求客户端向服务器发送请求
     * @param response 响应服务器向客户端发送的响应
     * @throws IOException 如果发生错误,引发IOException，则抛出ServletException。
     */
    public void doPost(HttpServletRequest request, HttpServletResponse response)
            throws IOException {

        System.out.println("进入AuthServlet");

        /**
        * 读取定义在web.xml文件中的全局参数DomainName，CookieName
        */
        DomainName= request.getSession().getServletContext().getInitParameter("DomainName");
        CookName = request.getSession().getServletContext().getInitParameter("CookieName");
        System.out.println(DomainName+"----跳转了----"+CookName);

        //获取在image.jsp中保存的验证码
        String ccode =(String) request.getSession().getAttribute("ccode");
        //获取在login.jsp中输入的验证码
        String checkcode = new String(request.getParameter("checkcode").getBytes(StandardCharsets.ISO_8859_1), StandardCharsets.UTF_8);
        //获取login.jsp的请求数据，location为带参数的登录页面url路径
        String location = request.getParameter("location");
        //去除url路径后面的/
        location=location.substring(0,location.length() -1);
        //判断验证码是否相等
        if(!checkcode.equals(ccode)){
            //如果不相等，则在控制台打印信息
            System.out.println("验证码错误");
            //重定向到登录页面
            response.sendRedirect(location);
        }else{
            //如果验证码正确，则判断用户名，密码是否相等
            //获取login.jsp中输入的用户名
            String username =request.getParameter("username");
            //获取login.jsp中输入的密码
            String userpassword =request.getParameter("password");
            //获取accounts模拟数据库中保存的用户名
            String key =accounts.get(username);
            //如果找不到该名用户，返回null
            if(key==null){
                System.out.println("没有此名用户");
                //重定向到登录页面
                response.sendRedirect(location);
            }else{
                //如果有该名用户，则判断密码
                if(key.equals(userpassword)){
                    //密码正确
                    //获取从login.jsp跳转到AuthServlet时所带的goto参数
                    String gotoURL = request.getParameter("goto");
                    //获得该方法返回一个包含分配给该 session会话的唯一标识符的字符串。
                    String sessionId =request.getSession().getId();
                    //创建了一个Cookie对象cookie
                    Cookie cookie =new Cookie(CookName,sessionId);
                    //设置最长生命周期
                    cookie.setMaxAge(100);
                    //解决跨域问题
                    cookie.setPath("/");
                    //将其加入到了HttpResponse对象
                    response.addCookie(cookie);
                    if (gotoURL != null) {
                        //这是直接从login.jsp页面访问，没有从index页面跳转过来的情况
                        if (gotoURL.equals("null/")){
                            //直接给这个情况绑定跳转到第一个欢迎页面(注意此时已经有Cookie了)
                            response.sendRedirect("http://localhost:8081/SSOWebDemo1/");
                            System.out.println("登录成功！！！！\n" + cookie + "----------------" + sessionId);
                        }else {
                            //其他正常的跳转情况
                            response.sendRedirect(gotoURL);
                            System.out.println("登录成功！！！！\n" + cookie + "----------------" + sessionId);
                        }
                    }else{
                        //goto参数为null的情况
                        System.out.println("重定向的网页不存在");
                        response.sendRedirect(location);
                    }
                }else{
                    //密码错误的情况
                    System.out.println("密码错误");
                    //重定向登录页面
                    response.sendRedirect(location);
                }
            }
        }
    }

    /**
     * accounts模拟数据库储存的用户数据
     * 添加了静态私有参数: 密钥名称CookName，域名DomainName
     * 初始化Servlet时，给系统添加了三个合法的用户
     * @param config  初始化参数
     */
    static private ConcurrentMap<String, String> accounts;
    String CookName;
    String DomainName;

    @Override
    public void init(ServletConfig config) {
        accounts= new ConcurrentHashMap<>();
        accounts.put("xzh", "123456");
        accounts.put("zzy", "123456");
        accounts.put("zsy", "123456");
    }

}