package org.servlet;

import java.io.IOException;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @项目: SSOWebDemo1
 * @实现: SSOAuth作为身份验证系统单独部署在了一个tomcat上，SSOWebDemo1和SSOWebDemo2是部署在同一个tomcat上。
 * @创建人: ZS
 * @创建时间: 2020/12/24
 * @描述: OutServlet:退出登录
 */
public class OutServlet extends HttpServlet {

    /**
     *调用父类的构造方法，这里调用了无参的构造方法
     */
    public OutServlet() {
        super();
    }

    /**
     *调用父类的销毁方法,关闭tomcat时销毁servlet
     */
    public void destroy() {
        super.destroy();
        System.out.println("销毁Outservlet");
    }

    /**
     * Servlet的doGet方法。
     * 当表单的标记值方法等于get时，将调用此方法。
     * @param request 请求客户端向服务器发送请求
     * @param response 响应服务器向客户端发送的响应
     * @throws IOException 如果发生错误,引发IOException，则抛出ServletException。
     */
    public void doGet(HttpServletRequest request, HttpServletResponse response)
            throws IOException {
        doPost(request,response);
    }

    /**
     * Servlet的doPost方法。
     * 当表单的标记值方法等于post时，将调用此方法。
     * @param request 请求客户端向服务器发送请求
     * @param response 响应服务器向客户端发送的响应
     * @throws IOException 如果发生错误,引发IOException，则抛出ServletException。
     */
    public void doPost(HttpServletRequest request, HttpServletResponse response)
            throws IOException {
        System.out.println("进入OutServlet");
        //获取从index.jsp跳转到OutServlet时所带的goto参数
        String url =request.getParameter("goto");

        //读取定义在web.xml文件中的全局参数CookeieName
        String CookieName =request.getSession().getServletContext().getInitParameter("CookieName");

        //退出登录，删除index页面的cookie
        try {
                //创建一个空的Cookie对象
                Cookie cookie = new Cookie(CookieName, null);
                //设置生命周期为0，相当于删除它
                cookie.setMaxAge(0);
                //解决跨域，根据你创建cookie的路径进行填写
                cookie.setPath("/");
                //将其加入到了HttpResponse对象
                response.addCookie(cookie);
                System.out.println("清空Cookie");
        }catch(Exception ex) {
            //不对则抛出异常
            System.out.println("清空Cookies发生异常！");
        }
        //最后重定向到欢迎页面index.jsp
        //进入index.jsp后会进入AuthServlet进行判断，如果cookie删除成功为空，则会跳转到登录页面
        response.sendRedirect(url);
    }
}